# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/agent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/agent.proto',
  package='cogmentAPI',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17\x63ogment/api/agent.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto2\xa5\x01\n\x0eServiceActorSP\x12Q\n\x08RunTrial\x12\x1e.cogmentAPI.ActorRunTrialInput\x1a\x1f.cogmentAPI.ActorRunTrialOutput\"\x00(\x01\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x62\x06proto3'
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_SERVICEACTORSP = _descriptor.ServiceDescriptor(
  name='ServiceActorSP',
  full_name='cogmentAPI.ServiceActorSP',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=66,
  serialized_end=231,
  methods=[
  _descriptor.MethodDescriptor(
    name='RunTrial',
    full_name='cogmentAPI.ServiceActorSP.RunTrial',
    index=0,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._ACTORRUNTRIALINPUT,
    output_type=cogment_dot_api_dot_common__pb2._ACTORRUNTRIALOUTPUT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='cogmentAPI.ServiceActorSP.Version',
    index=1,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._VERSIONREQUEST,
    output_type=cogment_dot_api_dot_common__pb2._VERSIONINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SERVICEACTORSP)

DESCRIPTOR.services_by_name['ServiceActorSP'] = _SERVICEACTORSP

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/common.proto',
  package='cogmentAPI',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x18\x63ogment/api/common.proto\x12\ncogmentAPI\x1a\x19google/protobuf/any.proto\"\x10\n\x0eVersionRequest\"j\n\x0bVersionInfo\x12\x31\n\x08versions\x18\x01 \x03(\x0b\x32\x1f.cogmentAPI.VersionInfo.Version\x1a(\n\x07Version\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"\x1e\n\x0bTrialConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"\x1e\n\x0b\x41\x63torConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"$\n\x11\x45nvironmentConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"9\n\rDatalogParams\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x16\n\x0e\x65xclude_fields\x18\x02 \x03(\t\"z\n\x11\x45nvironmentParams\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x02 \x01(\t\x12\x16\n\x0eimplementation\x18\x03 \x01(\t\x12-\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x1d.cogmentAPI.EnvironmentConfig\"\x83\x01\n\x0b\x41\x63torParams\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x61\x63tor_class\x18\x02 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x03 \x01(\t\x12\x16\n\x0eimplementation\x18\x04 \x01(\t\x12\'\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x17.cogmentAPI.ActorConfig\"\xf0\x01\n\x0bTrialParams\x12-\n\x0ctrial_config\x18\x01 \x01(\x0b\x32\x17.cogmentAPI.TrialConfig\x12*\n\x07\x64\x61talog\x18\x02 \x01(\x0b\x32\x19.cogmentAPI.DatalogParams\x12\x32\n\x0b\x65nvironment\x18\x03 \x01(\x0b\x32\x1d.cogmentAPI.EnvironmentParams\x12\'\n\x06\x61\x63tors\x18\x04 \x03(\x0b\x32\x17.cogmentAPI.ActorParams\x12\x11\n\tmax_steps\x18\x05 \x01(\r\x12\x16\n\x0emax_inactivity\x18\x06 \x01(\r\"/\n\nTrialActor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x61\x63tor_class\x18\x02 \x01(\t\"B\n\x0bObservation\x12\x0f\n\x07tick_id\x18\x01 \x01(\x04\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\"=\n\x06\x41\x63tion\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\"o\n\x0cRewardSource\x12\x13\n\x0bsender_name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02\x12\x12\n\nconfidence\x18\x03 \x01(\x02\x12\'\n\tuser_data\x18\x04 \x01(\x0b\x32\x14.google.protobuf.Any\"j\n\x06Reward\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x15\n\rreceiver_name\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x02\x12)\n\x07sources\x18\x04 \x03(\x0b\x32\x18.cogmentAPI.RewardSource\"m\n\x07Message\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x13\n\x0bsender_name\x18\x02 \x01(\t\x12\x15\n\rreceiver_name\x18\x03 \x01(\t\x12%\n\x07payload\x18\x04 \x01(\x0b\x32\x14.google.protobuf.Any\"\x91\x02\n\x12\x41\x63torRunTrialInput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x33\n\ninit_input\x18\x02 \x01(\x0b\x32\x1d.cogmentAPI.ActorInitialInputH\x00\x12.\n\x0bobservation\x18\x03 \x01(\x0b\x32\x17.cogmentAPI.ObservationH\x00\x12$\n\x06reward\x18\x04 \x01(\x0b\x32\x12.cogmentAPI.RewardH\x00\x12&\n\x07message\x18\x05 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x06 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\x8a\x02\n\x13\x41\x63torRunTrialOutput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x35\n\x0binit_output\x18\x02 \x01(\x0b\x32\x1e.cogmentAPI.ActorInitialOutputH\x00\x12$\n\x06\x61\x63tion\x18\x03 \x01(\x0b\x32\x12.cogmentAPI.ActionH\x00\x12$\n\x06reward\x18\x04 \x01(\x0b\x32\x12.cogmentAPI.RewardH\x00\x12&\n\x07message\x18\x05 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x06 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\x8a\x01\n\x11\x41\x63torInitialInput\x12\x12\n\nactor_name\x18\x01 \x01(\t\x12\x13\n\x0b\x61\x63tor_class\x18\x02 \x01(\t\x12\x11\n\timpl_name\x18\x03 \x01(\t\x12\x10\n\x08\x65nv_name\x18\x04 \x01(\t\x12\'\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x17.cogmentAPI.ActorConfig\"S\n\x12\x41\x63torInitialOutput\x12\x15\n\x0b\x61\x63tor_class\x18\x01 \x01(\tH\x00\x12\x14\n\nactor_name\x18\x02 \x01(\tH\x00\x42\x10\n\x0eslot_selection*a\n\nTrialState\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0cINITIALIZING\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x12\x0b\n\x07RUNNING\x10\x03\x12\x0f\n\x0bTERMINATING\x10\x04\x12\t\n\x05\x45NDED\x10\x05*g\n\x12\x43ommunicationState\x12\x15\n\x11UNKNOWN_COM_STATE\x10\x00\x12\n\n\x06NORMAL\x10\x01\x12\r\n\tHEARTBEAT\x10\x02\x12\x08\n\x04LAST\x10\x03\x12\x0c\n\x08LAST_ACK\x10\x04\x12\x07\n\x03\x45ND\x10\x05\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])

_TRIALSTATE = _descriptor.EnumDescriptor(
  name='TrialState',
  full_name='cogmentAPI.TrialState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INITIALIZING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RUNNING', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TERMINATING', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ENDED', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2138,
  serialized_end=2235,
)
_sym_db.RegisterEnumDescriptor(_TRIALSTATE)

TrialState = enum_type_wrapper.EnumTypeWrapper(_TRIALSTATE)
_COMMUNICATIONSTATE = _descriptor.EnumDescriptor(
  name='CommunicationState',
  full_name='cogmentAPI.CommunicationState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_COM_STATE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NORMAL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HEARTBEAT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LAST', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LAST_ACK', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='END', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2237,
  serialized_end=2340,
)
_sym_db.RegisterEnumDescriptor(_COMMUNICATIONSTATE)

CommunicationState = enum_type_wrapper.EnumTypeWrapper(_COMMUNICATIONSTATE)
UNKNOWN = 0
INITIALIZING = 1
PENDING = 2
RUNNING = 3
TERMINATING = 4
ENDED = 5
UNKNOWN_COM_STATE = 0
NORMAL = 1
HEARTBEAT = 2
LAST = 3
LAST_ACK = 4
END = 5



_VERSIONREQUEST = _descriptor.Descriptor(
  name='VersionRequest',
  full_name='cogmentAPI.VersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=83,
)


_VERSIONINFO_VERSION = _descriptor.Descriptor(
  name='Version',
  full_name='cogmentAPI.VersionInfo.Version',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='cogmentAPI.VersionInfo.Version.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='cogmentAPI.VersionInfo.Version.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=151,
  serialized_end=191,
)

_VERSIONINFO = _descriptor.Descriptor(
  name='VersionInfo',
  full_name='cogmentAPI.VersionInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='versions', full_name='cogmentAPI.VersionInfo.versions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_VERSIONINFO_VERSION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=191,
)


_TRIALCONFIG = _descriptor.Descriptor(
  name='TrialConfig',
  full_name='cogmentAPI.TrialConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogmentAPI.TrialConfig.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=193,
  serialized_end=223,
)


_ACTORCONFIG = _descriptor.Descriptor(
  name='ActorConfig',
  full_name='cogmentAPI.ActorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogmentAPI.ActorConfig.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=225,
  serialized_end=255,
)


_ENVIRONMENTCONFIG = _descriptor.Descriptor(
  name='EnvironmentConfig',
  full_name='cogmentAPI.EnvironmentConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cogmentAPI.EnvironmentConfig.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=257,
  serialized_end=293,
)


_DATALOGPARAMS = _descriptor.Descriptor(
  name='DatalogParams',
  full_name='cogmentAPI.DatalogParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='cogmentAPI.DatalogParams.endpoint', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exclude_fields', full_name='cogmentAPI.DatalogParams.exclude_fields', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=295,
  serialized_end=352,
)


_ENVIRONMENTPARAMS = _descriptor.Descriptor(
  name='EnvironmentParams',
  full_name='cogmentAPI.EnvironmentParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='cogmentAPI.EnvironmentParams.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='cogmentAPI.EnvironmentParams.endpoint', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='implementation', full_name='cogmentAPI.EnvironmentParams.implementation', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogmentAPI.EnvironmentParams.config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=354,
  serialized_end=476,
)


_ACTORPARAMS = _descriptor.Descriptor(
  name='ActorParams',
  full_name='cogmentAPI.ActorParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='cogmentAPI.ActorParams.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_class', full_name='cogmentAPI.ActorParams.actor_class', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='cogmentAPI.ActorParams.endpoint', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='implementation', full_name='cogmentAPI.ActorParams.implementation', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogmentAPI.ActorParams.config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=479,
  serialized_end=610,
)


_TRIALPARAMS = _descriptor.Descriptor(
  name='TrialParams',
  full_name='cogmentAPI.TrialParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trial_config', full_name='cogmentAPI.TrialParams.trial_config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datalog', full_name='cogmentAPI.TrialParams.datalog', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment', full_name='cogmentAPI.TrialParams.environment', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actors', full_name='cogmentAPI.TrialParams.actors', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_steps', full_name='cogmentAPI.TrialParams.max_steps', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_inactivity', full_name='cogmentAPI.TrialParams.max_inactivity', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=613,
  serialized_end=853,
)


_TRIALACTOR = _descriptor.Descriptor(
  name='TrialActor',
  full_name='cogmentAPI.TrialActor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='cogmentAPI.TrialActor.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_class', full_name='cogmentAPI.TrialActor.actor_class', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=855,
  serialized_end=902,
)


_OBSERVATION = _descriptor.Descriptor(
  name='Observation',
  full_name='cogmentAPI.Observation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.Observation.tick_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogmentAPI.Observation.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content', full_name='cogmentAPI.Observation.content', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=904,
  serialized_end=970,
)


_ACTION = _descriptor.Descriptor(
  name='Action',
  full_name='cogmentAPI.Action',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.Action.tick_id', index=0,
      number=1, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogmentAPI.Action.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content', full_name='cogmentAPI.Action.content', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=972,
  serialized_end=1033,
)


_REWARDSOURCE = _descriptor.Descriptor(
  name='RewardSource',
  full_name='cogmentAPI.RewardSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender_name', full_name='cogmentAPI.RewardSource.sender_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='cogmentAPI.RewardSource.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='cogmentAPI.RewardSource.confidence', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_data', full_name='cogmentAPI.RewardSource.user_data', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1035,
  serialized_end=1146,
)


_REWARD = _descriptor.Descriptor(
  name='Reward',
  full_name='cogmentAPI.Reward',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.Reward.tick_id', index=0,
      number=1, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='receiver_name', full_name='cogmentAPI.Reward.receiver_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='cogmentAPI.Reward.value', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sources', full_name='cogmentAPI.Reward.sources', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1148,
  serialized_end=1254,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='cogmentAPI.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.Message.tick_id', index=0,
      number=1, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender_name', full_name='cogmentAPI.Message.sender_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='receiver_name', full_name='cogmentAPI.Message.receiver_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='cogmentAPI.Message.payload', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1256,
  serialized_end=1365,
)


_ACTORRUNTRIALINPUT = _descriptor.Descriptor(
  name='ActorRunTrialInput',
  full_name='cogmentAPI.ActorRunTrialInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='cogmentAPI.ActorRunTrialInput.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='init_input', full_name='cogmentAPI.ActorRunTrialInput.init_input', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation', full_name='cogmentAPI.ActorRunTrialInput.observation', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reward', full_name='cogmentAPI.ActorRunTrialInput.reward', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='cogmentAPI.ActorRunTrialInput.message', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='details', full_name='cogmentAPI.ActorRunTrialInput.details', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='cogmentAPI.ActorRunTrialInput.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1368,
  serialized_end=1641,
)


_ACTORRUNTRIALOUTPUT = _descriptor.Descriptor(
  name='ActorRunTrialOutput',
  full_name='cogmentAPI.ActorRunTrialOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='cogmentAPI.ActorRunTrialOutput.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='init_output', full_name='cogmentAPI.ActorRunTrialOutput.init_output', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action', full_name='cogmentAPI.ActorRunTrialOutput.action', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reward', full_name='cogmentAPI.ActorRunTrialOutput.reward', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='cogmentAPI.ActorRunTrialOutput.message', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='details', full_name='cogmentAPI.ActorRunTrialOutput.details', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='cogmentAPI.ActorRunTrialOutput.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1644,
  serialized_end=1910,
)


_ACTORINITIALINPUT = _descriptor.Descriptor(
  name='ActorInitialInput',
  full_name='cogmentAPI.ActorInitialInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='actor_name', full_name='cogmentAPI.ActorInitialInput.actor_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_class', full_name='cogmentAPI.ActorInitialInput.actor_class', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='impl_name', full_name='cogmentAPI.ActorInitialInput.impl_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='env_name', full_name='cogmentAPI.ActorInitialInput.env_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogmentAPI.ActorInitialInput.config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1913,
  serialized_end=2051,
)


_ACTORINITIALOUTPUT = _descriptor.Descriptor(
  name='ActorInitialOutput',
  full_name='cogmentAPI.ActorInitialOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='actor_class', full_name='cogmentAPI.ActorInitialOutput.actor_class', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actor_name', full_name='cogmentAPI.ActorInitialOutput.actor_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='slot_selection', full_name='cogmentAPI.ActorInitialOutput.slot_selection',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2053,
  serialized_end=2136,
)

_VERSIONINFO_VERSION.containing_type = _VERSIONINFO
_VERSIONINFO.fields_by_name['versions'].message_type = _VERSIONINFO_VERSION
_ENVIRONMENTPARAMS.fields_by_name['config'].message_type = _ENVIRONMENTCONFIG
_ACTORPARAMS.fields_by_name['config'].message_type = _ACTORCONFIG
_TRIALPARAMS.fields_by_name['trial_config'].message_type = _TRIALCONFIG
_TRIALPARAMS.fields_by_name['datalog'].message_type = _DATALOGPARAMS
_TRIALPARAMS.fields_by_name['environment'].message_type = _ENVIRONMENTPARAMS
_TRIALPARAMS.fields_by_name['actors'].message_type = _ACTORPARAMS
_REWARDSOURCE.fields_by_name['user_data'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_REWARD.fields_by_name['sources'].message_type = _REWARDSOURCE
_MESSAGE.fields_by_name['payload'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_ACTORRUNTRIALINPUT.fields_by_name['state'].enum_type = _COMMUNICATIONSTATE
_ACTORRUNTRIALINPUT.fields_by_name['init_input'].message_type = _ACTORINITIALINPUT
_ACTORRUNTRIALINPUT.fields_by_name['observation'].message_type = _OBSERVATION
_ACTORRUNTRIALINPUT.fields_by_name['reward'].message_type = _REWARD
_ACTORRUNTRIALINPUT.fields_by_name['message'].message_type = _MESSAGE
_ACTORRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALINPUT.fields_by_name['init_input'])
_ACTORRUNTRIALINPUT.fields_by_name['init_input'].containing_oneof = _ACTORRUNTRIALINPUT.oneofs_by_name['data']
_ACTORRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALINPUT.fields_by_name['observation'])
_ACTORRUNTRIALINPUT.fields_by_name['observation'].containing_oneof = _ACTORRUNTRIALINPUT.oneofs_by_name['data']
_ACTORRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALINPUT.fields_by_name['reward'])
_ACTORRUNTRIALINPUT.fields_by_name['reward'].containing_oneof = _ACTORRUNTRIALINPUT.oneofs_by_name['data']
_ACTORRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALINPUT.fields_by_name['message'])
_ACTORRUNTRIALINPUT.fields_by_name['message'].containing_oneof = _ACTORRUNTRIALINPUT.oneofs_by_name['data']
_ACTORRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALINPUT.fields_by_name['details'])
_ACTORRUNTRIALINPUT.fields_by_name['details'].containing_oneof = _ACTORRUNTRIALINPUT.oneofs_by_name['data']
_ACTORRUNTRIALOUTPUT.fields_by_name['state'].enum_type = _COMMUNICATIONSTATE
_ACTORRUNTRIALOUTPUT.fields_by_name['init_output'].message_type = _ACTORINITIALOUTPUT
_ACTORRUNTRIALOUTPUT.fields_by_name['action'].message_type = _ACTION
_ACTORRUNTRIALOUTPUT.fields_by_name['reward'].message_type = _REWARD
_ACTORRUNTRIALOUTPUT.fields_by_name['message'].message_type = _MESSAGE
_ACTORRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALOUTPUT.fields_by_name['init_output'])
_ACTORRUNTRIALOUTPUT.fields_by_name['init_output'].containing_oneof = _ACTORRUNTRIALOUTPUT.oneofs_by_name['data']
_ACTORRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALOUTPUT.fields_by_name['action'])
_ACTORRUNTRIALOUTPUT.fields_by_name['action'].containing_oneof = _ACTORRUNTRIALOUTPUT.oneofs_by_name['data']
_ACTORRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALOUTPUT.fields_by_name['reward'])
_ACTORRUNTRIALOUTPUT.fields_by_name['reward'].containing_oneof = _ACTORRUNTRIALOUTPUT.oneofs_by_name['data']
_ACTORRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALOUTPUT.fields_by_name['message'])
_ACTORRUNTRIALOUTPUT.fields_by_name['message'].containing_oneof = _ACTORRUNTRIALOUTPUT.oneofs_by_name['data']
_ACTORRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ACTORRUNTRIALOUTPUT.fields_by_name['details'])
_ACTORRUNTRIALOUTPUT.fields_by_name['details'].containing_oneof = _ACTORRUNTRIALOUTPUT.oneofs_by_name['data']
_ACTORINITIALINPUT.fields_by_name['config'].message_type = _ACTORCONFIG
_ACTORINITIALOUTPUT.oneofs_by_name['slot_selection'].fields.append(
  _ACTORINITIALOUTPUT.fields_by_name['actor_class'])
_ACTORINITIALOUTPUT.fields_by_name['actor_class'].containing_oneof = _ACTORINITIALOUTPUT.oneofs_by_name['slot_selection']
_ACTORINITIALOUTPUT.oneofs_by_name['slot_selection'].fields.append(
  _ACTORINITIALOUTPUT.fields_by_name['actor_name'])
_ACTORINITIALOUTPUT.fields_by_name['actor_name'].containing_oneof = _ACTORINITIALOUTPUT.oneofs_by_name['slot_selection']
DESCRIPTOR.message_types_by_name['VersionRequest'] = _VERSIONREQUEST
DESCRIPTOR.message_types_by_name['VersionInfo'] = _VERSIONINFO
DESCRIPTOR.message_types_by_name['TrialConfig'] = _TRIALCONFIG
DESCRIPTOR.message_types_by_name['ActorConfig'] = _ACTORCONFIG
DESCRIPTOR.message_types_by_name['EnvironmentConfig'] = _ENVIRONMENTCONFIG
DESCRIPTOR.message_types_by_name['DatalogParams'] = _DATALOGPARAMS
DESCRIPTOR.message_types_by_name['EnvironmentParams'] = _ENVIRONMENTPARAMS
DESCRIPTOR.message_types_by_name['ActorParams'] = _ACTORPARAMS
DESCRIPTOR.message_types_by_name['TrialParams'] = _TRIALPARAMS
DESCRIPTOR.message_types_by_name['TrialActor'] = _TRIALACTOR
DESCRIPTOR.message_types_by_name['Observation'] = _OBSERVATION
DESCRIPTOR.message_types_by_name['Action'] = _ACTION
DESCRIPTOR.message_types_by_name['RewardSource'] = _REWARDSOURCE
DESCRIPTOR.message_types_by_name['Reward'] = _REWARD
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.message_types_by_name['ActorRunTrialInput'] = _ACTORRUNTRIALINPUT
DESCRIPTOR.message_types_by_name['ActorRunTrialOutput'] = _ACTORRUNTRIALOUTPUT
DESCRIPTOR.message_types_by_name['ActorInitialInput'] = _ACTORINITIALINPUT
DESCRIPTOR.message_types_by_name['ActorInitialOutput'] = _ACTORINITIALOUTPUT
DESCRIPTOR.enum_types_by_name['TrialState'] = _TRIALSTATE
DESCRIPTOR.enum_types_by_name['CommunicationState'] = _COMMUNICATIONSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VersionRequest = _reflection.GeneratedProtocolMessageType('VersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERSIONREQUEST,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.VersionRequest)
  })
_sym_db.RegisterMessage(VersionRequest)

VersionInfo = _reflection.GeneratedProtocolMessageType('VersionInfo', (_message.Message,), {

  'Version' : _reflection.GeneratedProtocolMessageType('Version', (_message.Message,), {
    'DESCRIPTOR' : _VERSIONINFO_VERSION,
    '__module__' : 'cogment.api.common_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.VersionInfo.Version)
    })
  ,
  'DESCRIPTOR' : _VERSIONINFO,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.VersionInfo)
  })
_sym_db.RegisterMessage(VersionInfo)
_sym_db.RegisterMessage(VersionInfo.Version)

TrialConfig = _reflection.GeneratedProtocolMessageType('TrialConfig', (_message.Message,), {
  'DESCRIPTOR' : _TRIALCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialConfig)
  })
_sym_db.RegisterMessage(TrialConfig)

ActorConfig = _reflection.GeneratedProtocolMessageType('ActorConfig', (_message.Message,), {
  'DESCRIPTOR' : _ACTORCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorConfig)
  })
_sym_db.RegisterMessage(ActorConfig)

EnvironmentConfig = _reflection.GeneratedProtocolMessageType('EnvironmentConfig', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTCONFIG,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvironmentConfig)
  })
_sym_db.RegisterMessage(EnvironmentConfig)

DatalogParams = _reflection.GeneratedProtocolMessageType('DatalogParams', (_message.Message,), {
  'DESCRIPTOR' : _DATALOGPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DatalogParams)
  })
_sym_db.RegisterMessage(DatalogParams)

EnvironmentParams = _reflection.GeneratedProtocolMessageType('EnvironmentParams', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvironmentParams)
  })
_sym_db.RegisterMessage(EnvironmentParams)

ActorParams = _reflection.GeneratedProtocolMessageType('ActorParams', (_message.Message,), {
  'DESCRIPTOR' : _ACTORPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorParams)
  })
_sym_db.RegisterMessage(ActorParams)

TrialParams = _reflection.GeneratedProtocolMessageType('TrialParams', (_message.Message,), {
  'DESCRIPTOR' : _TRIALPARAMS,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialParams)
  })
_sym_db.RegisterMessage(TrialParams)

TrialActor = _reflection.GeneratedProtocolMessageType('TrialActor', (_message.Message,), {
  'DESCRIPTOR' : _TRIALACTOR,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialActor)
  })
_sym_db.RegisterMessage(TrialActor)

Observation = _reflection.GeneratedProtocolMessageType('Observation', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Observation)
  })
_sym_db.RegisterMessage(Observation)

Action = _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
  'DESCRIPTOR' : _ACTION,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Action)
  })
_sym_db.RegisterMessage(Action)

RewardSource = _reflection.GeneratedProtocolMessageType('RewardSource', (_message.Message,), {
  'DESCRIPTOR' : _REWARDSOURCE,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RewardSource)
  })
_sym_db.RegisterMessage(RewardSource)

Reward = _reflection.GeneratedProtocolMessageType('Reward', (_message.Message,), {
  'DESCRIPTOR' : _REWARD,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Reward)
  })
_sym_db.RegisterMessage(Reward)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.Message)
  })
_sym_db.RegisterMessage(Message)

ActorRunTrialInput = _reflection.GeneratedProtocolMessageType('ActorRunTrialInput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORRUNTRIALINPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorRunTrialInput)
  })
_sym_db.RegisterMessage(ActorRunTrialInput)

ActorRunTrialOutput = _reflection.GeneratedProtocolMessageType('ActorRunTrialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORRUNTRIALOUTPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorRunTrialOutput)
  })
_sym_db.RegisterMessage(ActorRunTrialOutput)

ActorInitialInput = _reflection.GeneratedProtocolMessageType('ActorInitialInput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORINITIALINPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorInitialInput)
  })
_sym_db.RegisterMessage(ActorInitialInput)

ActorInitialOutput = _reflection.GeneratedProtocolMessageType('ActorInitialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ACTORINITIALOUTPUT,
  '__module__' : 'cogment.api.common_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActorInitialOutput)
  })
_sym_db.RegisterMessage(ActorInitialOutput)


# @@protoc_insertion_point(module_scope)



# Copyright 2021 AI Redefined Inc. <dev+cogment@ai-r.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import grpc

import cogment.api.common_pb2 as common_api

from cogment.version import __version__


# logging level for "trace" (i.e. repeated output in critical path) since Python logging does not have a TRACE level
# Use: logging.log(TRACE, f"This is a trace message for {my_pgm}")
TRACE = 5

# Timeout (in seconds) for the init data to come from the Orchestrator before we consider a failure.
INIT_TIMEOUT = 30


def list_versions():
    reply = common_api.VersionInfo()
    reply.versions.add(name='cogment_sdk', version=__version__)
    reply.versions.add(name='grpc', version=grpc.__version__)

    return reply



# Copyright 2021 AI Redefined Inc. <dev+cogment@ai-r.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from posixpath import join
import grpc
import grpc.aio  # type: ignore
from prometheus_client import start_http_server as start_prometheus_server
from prometheus_client.core import REGISTRY  # type: ignore

import cogment.api.orchestrator_pb2_grpc as orchestrator_grpc_api
import cogment.api.agent_pb2_grpc as agent_grpc_api
import cogment.api.environment_pb2_grpc as env_grpc_api
import cogment.api.hooks_pb2_grpc as hooks_grpc_api
import cogment.api.datalog_pb2_grpc as datalog_grpc_api
import cogment.api.agent_pb2 as agent_api

from cogment.actor import ActorSession
from cogment.environment import EnvironmentSession
from cogment.prehook import PrehookSession
from cogment.datalog import DatalogSession
from cogment.control import Controller
from cogment.errors import CogmentError
from cogment.agent_service import AgentServicer, get_actor_impl
from cogment.client_service import ClientServicer
from cogment.env_service import EnvironmentServicer
from cogment.hooks_service import PrehookServicer
from cogment.datalog_service import DatalogServicer

import logging
from typing import Callable, Awaitable, Dict, List, Any
from types import ModuleType
import os
import asyncio
from types import SimpleNamespace


DEFAULT_PROMETHEUS_PORT = 8000


class Endpoint:
    """Class representing a remote Cogment endpoint where to connect."""

    def __init__(self, url: str):
        self.url = url
        self.private_key = None
        self.root_certificates = None
        self.certificate_chain = None

    def __str__(self):
        result = f"Endpoint: url = {self.url}, private_key = {self.private_key}"
        result += f", root_certificates = {self.root_certificates}, certificate_chain = {self.certificate_chain}"
        return result

    def set_from_files(self, private_key_file=None, root_certificates_file=None, certificate_chain_file=None):
        try:
            if private_key_file:
                with open(private_key_file) as fl:
                    self.private_key = fl.read()
                if not self.private_key:
                    self.private_key = None

            if root_certificates_file:
                with open(root_certificates_file) as fl:
                    self.root_certificates = fl.read()
                if not self.root_certificates:
                    self.root_certificates = None

            if certificate_chain_file:
                with open(certificate_chain_file) as fl:
                    self.certificate_chain = fl.read()
                if not self.certificate_chain:
                    self.certificate_chain = None

        except Exception:
            logging.error(f"Failed loading file from CWD: {os.getcwd()}")
            raise


class ServedEndpoint:
    """Class representing a local Cogment endpoint where others can connect."""

    def __init__(self, port: int):
        self.port = port
        self.private_key_certificate_chain_pairs = None
        self.root_certificates = None

    # def set_from_files(private_key_certificate_chain_pairs_file=None, root_certificates_file=None):
    # TODO: This function would need to parse the PEM encoded `private_key_certificate_chain_pairs_file`
    #       to create the list of tuples required (see simpler version in `Endpoint` class above).

    def __str__(self):
        result = f"ServedEndpoint: port = {self.port}"
        result += f", private_key_certificate_chain_pairs = {self.private_key_certificate_chain_pairs}"
        result += f", root_certificates = {self.root_certificates}"
        return result


def _make_client_channel(endpoint: Endpoint):
    if endpoint.url[:7] == "grpc://":
        url = endpoint.url[7:]
    else:
        logging.warning(f"Endpoint URL must be of gRPC type (start with 'grpc://') [{endpoint.url}]")
        url = endpoint.url

    if endpoint.private_key is None:
        channel = grpc.aio.insecure_channel(url)
    else:
        if endpoint.root_certificates:
            root = bytes(endpoint.root_certificates, "utf-8")
        else:
            root = None
        if endpoint.private_key:
            key = bytes(endpoint.private_key, "utf-8")
        else:
            key = None
        if endpoint.certificate_chain:
            certs = bytes(endpoint.certificate_chain, "utf-8")
        else:
            certs = None
        creds = grpc.ssl_channel_credentials(root, key, certs)
        channel = grpc.aio.secure_channel(url, creds)

    return channel


def _make_server(endpoint: ServedEndpoint):
    server = grpc.aio.server()

    address = f"[::]:{endpoint.port}"
    if endpoint.private_key_certificate_chain_pairs is None:
        server.add_insecure_port(address)
    else:
        if endpoint.root_certificates:
            require_client_auth = True
            root = bytes(endpoint.root_certificates, "utf-8")
        else:
            require_client_auth = False
            root = None
        certs = []
        for (key, chain) in endpoint.private_key_certificate_chain_pairs:
            certs.append((bytes(key, "utf-8"), bytes(chain, "utf-8")))
        if not certs:
            certs = None

        creds = grpc.ssl_server_credentials(certs, root, require_client_auth)
        server.add_secure_port(address, creds)

    return server


class Context:
    """Top level class for the Cogment library from which to obtain all services."""

    def __init__(self, user_id: str, cog_settings: ModuleType, asyncio_loop=None, prometheus_registry=REGISTRY):
        self._user_id = user_id
        self._actor_impls: Dict[str, SimpleNamespace] = {}
        self._env_impls: Dict[str, SimpleNamespace] = {}
        self._prehook_impl: Callable[[PrehookSession], Awaitable[None]] = None
        self._datalog_impl: Callable[[DatalogSession], Awaitable[None]] = None
        self._grpc_server = None  # type: Any
        self._prometheus_registry = prometheus_registry
        self._cog_settings = cog_settings

        if asyncio_loop is None:
            # Make sure we are running in a asyncio.Task. Even if technically this init does not need
            # to be in a Task, the whole of the SDK expects to be running in Tasks of the same event loop.
            try:
                self.asyncio_loop = asyncio.get_running_loop()
            except RuntimeError:
                raise CogmentError("The Cogment SDK requires running in a Python `asyncio` Task")
        else:
            self.asyncio_loop = asyncio_loop

    def __str__(self):
        result = f"Cogment Context: user id = {self._user_id}"
        return result

    def register_actor(self,
                       impl: Callable[[ActorSession], Awaitable[None]],
                       impl_name: str,
                       actor_classes: List[str] = []):
        if self._grpc_server is not None:
            # We could accept "client" actor registration after the server is started, but it is not worth it
            raise CogmentError("Cannot register an actor after the server is started")
        if impl_name in self._actor_impls:
            raise CogmentError(f"The actor implementation name must be unique: [{impl_name}]")

        self._actor_impls[impl_name] = SimpleNamespace(impl=impl, actor_classes=actor_classes)

    def register_environment(self,
                             impl: Callable[[EnvironmentSession], Awaitable[None]],
                             impl_name: str = "default"):
        if self._grpc_server is not None:
            raise CogmentError("Cannot register an environment after the server is started")
        if impl_name in self._env_impls:
            raise CogmentError(f"The environment implementation name must be unique: [{impl_name}]")

        self._env_impls[impl_name] = SimpleNamespace(impl=impl)

    def register_pre_trial_hook(self,
                                impl: Callable[[PrehookSession], Awaitable[None]]):
        if self._grpc_server is not None:
            raise CogmentError("Cannot register a pre-trial hook after the server is started")
        if self._prehook_impl is not None:
            raise CogmentError("Only one pre-trial hook service can be registered")

        self._prehook_impl = impl

    def register_datalog(self,
                         impl: Callable[[DatalogSession], Awaitable[None]]):
        if self._grpc_server is not None:
            raise CogmentError("Cannot register a datalog after the server is started")
        if self._datalog_impl is not None:
            raise CogmentError("Only one datalog service can be registered")

        self._datalog_impl = impl

    async def serve_all_registered(self, served_endpoint: ServedEndpoint, prometheus_port=DEFAULT_PROMETHEUS_PORT):
        if (len(self._actor_impls) == 0 and len(self._env_impls) == 0 and
                self._prehook_impl is None and self._datalog_impl is None):
            raise CogmentError("Nothing registered to serve!")
        if self._grpc_server is not None:
            raise CogmentError("Cannot serve the same components twice")

        self._grpc_server = _make_server(served_endpoint)

        if self._actor_impls:
            servicer = AgentServicer(self._actor_impls, self._cog_settings, self._prometheus_registry)
            agent_grpc_api.add_ServiceActorSPServicer_to_server(servicer, self._grpc_server)

        if self._env_impls:
            servicer = EnvironmentServicer(self._env_impls, self._cog_settings, self._prometheus_registry)
            env_grpc_api.add_EnvironmentSPServicer_to_server(servicer, self._grpc_server)

        if self._prehook_impl is not None:
            servicer = PrehookServicer(self._prehook_impl, self._cog_settings, self._prometheus_registry)
            hooks_grpc_api.add_TrialHooksSPServicer_to_server(servicer, self._grpc_server)

        if self._datalog_impl is not None:
            servicer = DatalogServicer(self._datalog_impl, self._cog_settings)
            datalog_grpc_api.add_DatalogSPServicer_to_server(servicer, self._grpc_server)

        if self._prometheus_registry is not None and prometheus_port is not None:
            start_prometheus_server(prometheus_port, "", self._prometheus_registry)

        await self._grpc_server.start()
        logging.debug(f"Context gRPC server at port [{served_endpoint.port}] for user [{self._user_id}] started")
        await self._grpc_server.wait_for_termination()
        logging.debug(f"Context gRPC server at port [{served_endpoint.port}] for user [{self._user_id}] exited")

    def get_controller(self, endpoint: Endpoint):
        channel = _make_client_channel(endpoint)
        stub = orchestrator_grpc_api.TrialLifecycleSPStub(channel)
        return Controller(stub, self._user_id)

    async def join_trial(self, trial_id, endpoint: Endpoint, impl_name=None, actor_name=None, actor_class=None):
        requested_class = None
        requested_name = None
        if impl_name is not None:
            # For backward compatibility
            logging.warning(f"`join_trial` parameter `impl_name` is deprecated")
            if actor_name is None:
                actor_impl = self._actor_impls[impl_name]
                if len(actor_impl.actor_classes) == 0:
                    raise CogmentError(f"Unable to determine possible actor to join trial: "
                                       f"impl_name [{impl_name}] does not have any registered actor class")
                requested_class = actor_impl.actor_classes[0]
            else:
                requested_name = actor_name

        elif actor_name is not None:
            requested_name = actor_name
            if actor_class is not None:
                logging.warning(f"`actor_class` will be ignored because `actor_name` is provided in `join_trial")

        elif actor_class is not None:
            requested_class = actor_class

        else:
            raise CogmentError(f"Actor name or actor class must be specified to join a trial")

        channel = _make_client_channel(endpoint)
        stub = orchestrator_grpc_api.ClientActorSPStub(channel)
        servicer = ClientServicer(self._cog_settings, stub)

        init_data = await servicer.join_trial(trial_id, requested_name, requested_class)
        if not init_data:
            return

        if requested_name is not None and requested_name != init_data.actor_name:
            raise CogmentError(f"Internal failure: Actor name [{requested_name}] requested, received: {init_data}")
        if requested_class is not None and requested_class != init_data.actor_class:
            raise CogmentError(f"Internal failure: Actor class [{requested_class}] requested, received: {init_data}")

        if impl_name is not None and init_data.impl_name and init_data.impl_name != impl_name:
            logging.warning(f"Requested impl_name [{impl_name}] does not match trial impl_name "
                            f"[{init_data.impl_name}]: Requested impl_name will be used.")
            init_data.impl_name = impl_name

        actor_impl = get_actor_impl(trial_id, self._actor_impls, init_data)

        await servicer.run_session(actor_impl.impl, init_data)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/environment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/environment.proto',
  package='cogmentAPI',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1d\x63ogment/api/environment.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"^\n\x0eObservationSet\x12\x0f\n\x07tick_id\x18\x01 \x01(\x12\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x14\n\x0cobservations\x18\x03 \x03(\x0c\x12\x12\n\nactors_map\x18\x04 \x03(\x05\"@\n\tActionSet\x12\x0f\n\x07tick_id\x18\x01 \x01(\x04\x12\x11\n\ttimestamp\x18\x02 \x01(\x06\x12\x0f\n\x07\x61\x63tions\x18\x03 \x03(\x0c\"\xe4\x01\n\x10\x45nvRunTrialInput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x31\n\ninit_input\x18\x02 \x01(\x0b\x32\x1b.cogmentAPI.EnvInitialInputH\x00\x12+\n\naction_set\x18\x03 \x01(\x0b\x32\x15.cogmentAPI.ActionSetH\x00\x12&\n\x07message\x18\x04 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x05 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\x97\x02\n\x11\x45nvRunTrialOutput\x12-\n\x05state\x18\x01 \x01(\x0e\x32\x1e.cogmentAPI.CommunicationState\x12\x33\n\x0binit_output\x18\x02 \x01(\x0b\x32\x1c.cogmentAPI.EnvInitialOutputH\x00\x12\x35\n\x0fobservation_set\x18\x03 \x01(\x0b\x32\x1a.cogmentAPI.ObservationSetH\x00\x12$\n\x06reward\x18\x04 \x01(\x0b\x32\x12.cogmentAPI.RewardH\x00\x12&\n\x07message\x18\x05 \x01(\x0b\x32\x13.cogmentAPI.MessageH\x00\x12\x11\n\x07\x64\x65tails\x18\x06 \x01(\tH\x00\x42\x06\n\x04\x64\x61ta\"\xa3\x01\n\x0f\x45nvInitialInput\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\timpl_name\x18\x02 \x01(\t\x12\x0f\n\x07tick_id\x18\x03 \x01(\x04\x12/\n\x0f\x61\x63tors_in_trial\x18\x04 \x03(\x0b\x32\x16.cogmentAPI.TrialActor\x12-\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x1d.cogmentAPI.EnvironmentConfig\"\x12\n\x10\x45nvInitialOutput2\xa0\x01\n\rEnvironmentSP\x12M\n\x08RunTrial\x12\x1c.cogmentAPI.EnvRunTrialInput\x1a\x1d.cogmentAPI.EnvRunTrialOutput\"\x00(\x01\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x62\x06proto3'
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,])




_OBSERVATIONSET = _descriptor.Descriptor(
  name='ObservationSet',
  full_name='cogmentAPI.ObservationSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.ObservationSet.tick_id', index=0,
      number=1, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogmentAPI.ObservationSet.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observations', full_name='cogmentAPI.ObservationSet.observations', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actors_map', full_name='cogmentAPI.ObservationSet.actors_map', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=165,
)


_ACTIONSET = _descriptor.Descriptor(
  name='ActionSet',
  full_name='cogmentAPI.ActionSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.ActionSet.tick_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='cogmentAPI.ActionSet.timestamp', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actions', full_name='cogmentAPI.ActionSet.actions', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=167,
  serialized_end=231,
)


_ENVRUNTRIALINPUT = _descriptor.Descriptor(
  name='EnvRunTrialInput',
  full_name='cogmentAPI.EnvRunTrialInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='cogmentAPI.EnvRunTrialInput.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='init_input', full_name='cogmentAPI.EnvRunTrialInput.init_input', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action_set', full_name='cogmentAPI.EnvRunTrialInput.action_set', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='cogmentAPI.EnvRunTrialInput.message', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='details', full_name='cogmentAPI.EnvRunTrialInput.details', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='cogmentAPI.EnvRunTrialInput.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=234,
  serialized_end=462,
)


_ENVRUNTRIALOUTPUT = _descriptor.Descriptor(
  name='EnvRunTrialOutput',
  full_name='cogmentAPI.EnvRunTrialOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='cogmentAPI.EnvRunTrialOutput.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='init_output', full_name='cogmentAPI.EnvRunTrialOutput.init_output', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_set', full_name='cogmentAPI.EnvRunTrialOutput.observation_set', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reward', full_name='cogmentAPI.EnvRunTrialOutput.reward', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='cogmentAPI.EnvRunTrialOutput.message', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='details', full_name='cogmentAPI.EnvRunTrialOutput.details', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='cogmentAPI.EnvRunTrialOutput.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=465,
  serialized_end=744,
)


_ENVINITIALINPUT = _descriptor.Descriptor(
  name='EnvInitialInput',
  full_name='cogmentAPI.EnvInitialInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='cogmentAPI.EnvInitialInput.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='impl_name', full_name='cogmentAPI.EnvInitialInput.impl_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_id', full_name='cogmentAPI.EnvInitialInput.tick_id', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='actors_in_trial', full_name='cogmentAPI.EnvInitialInput.actors_in_trial', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='config', full_name='cogmentAPI.EnvInitialInput.config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=747,
  serialized_end=910,
)


_ENVINITIALOUTPUT = _descriptor.Descriptor(
  name='EnvInitialOutput',
  full_name='cogmentAPI.EnvInitialOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=912,
  serialized_end=930,
)

_ENVRUNTRIALINPUT.fields_by_name['state'].enum_type = cogment_dot_api_dot_common__pb2._COMMUNICATIONSTATE
_ENVRUNTRIALINPUT.fields_by_name['init_input'].message_type = _ENVINITIALINPUT
_ENVRUNTRIALINPUT.fields_by_name['action_set'].message_type = _ACTIONSET
_ENVRUNTRIALINPUT.fields_by_name['message'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
_ENVRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALINPUT.fields_by_name['init_input'])
_ENVRUNTRIALINPUT.fields_by_name['init_input'].containing_oneof = _ENVRUNTRIALINPUT.oneofs_by_name['data']
_ENVRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALINPUT.fields_by_name['action_set'])
_ENVRUNTRIALINPUT.fields_by_name['action_set'].containing_oneof = _ENVRUNTRIALINPUT.oneofs_by_name['data']
_ENVRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALINPUT.fields_by_name['message'])
_ENVRUNTRIALINPUT.fields_by_name['message'].containing_oneof = _ENVRUNTRIALINPUT.oneofs_by_name['data']
_ENVRUNTRIALINPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALINPUT.fields_by_name['details'])
_ENVRUNTRIALINPUT.fields_by_name['details'].containing_oneof = _ENVRUNTRIALINPUT.oneofs_by_name['data']
_ENVRUNTRIALOUTPUT.fields_by_name['state'].enum_type = cogment_dot_api_dot_common__pb2._COMMUNICATIONSTATE
_ENVRUNTRIALOUTPUT.fields_by_name['init_output'].message_type = _ENVINITIALOUTPUT
_ENVRUNTRIALOUTPUT.fields_by_name['observation_set'].message_type = _OBSERVATIONSET
_ENVRUNTRIALOUTPUT.fields_by_name['reward'].message_type = cogment_dot_api_dot_common__pb2._REWARD
_ENVRUNTRIALOUTPUT.fields_by_name['message'].message_type = cogment_dot_api_dot_common__pb2._MESSAGE
_ENVRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALOUTPUT.fields_by_name['init_output'])
_ENVRUNTRIALOUTPUT.fields_by_name['init_output'].containing_oneof = _ENVRUNTRIALOUTPUT.oneofs_by_name['data']
_ENVRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALOUTPUT.fields_by_name['observation_set'])
_ENVRUNTRIALOUTPUT.fields_by_name['observation_set'].containing_oneof = _ENVRUNTRIALOUTPUT.oneofs_by_name['data']
_ENVRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALOUTPUT.fields_by_name['reward'])
_ENVRUNTRIALOUTPUT.fields_by_name['reward'].containing_oneof = _ENVRUNTRIALOUTPUT.oneofs_by_name['data']
_ENVRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALOUTPUT.fields_by_name['message'])
_ENVRUNTRIALOUTPUT.fields_by_name['message'].containing_oneof = _ENVRUNTRIALOUTPUT.oneofs_by_name['data']
_ENVRUNTRIALOUTPUT.oneofs_by_name['data'].fields.append(
  _ENVRUNTRIALOUTPUT.fields_by_name['details'])
_ENVRUNTRIALOUTPUT.fields_by_name['details'].containing_oneof = _ENVRUNTRIALOUTPUT.oneofs_by_name['data']
_ENVINITIALINPUT.fields_by_name['actors_in_trial'].message_type = cogment_dot_api_dot_common__pb2._TRIALACTOR
_ENVINITIALINPUT.fields_by_name['config'].message_type = cogment_dot_api_dot_common__pb2._ENVIRONMENTCONFIG
DESCRIPTOR.message_types_by_name['ObservationSet'] = _OBSERVATIONSET
DESCRIPTOR.message_types_by_name['ActionSet'] = _ACTIONSET
DESCRIPTOR.message_types_by_name['EnvRunTrialInput'] = _ENVRUNTRIALINPUT
DESCRIPTOR.message_types_by_name['EnvRunTrialOutput'] = _ENVRUNTRIALOUTPUT
DESCRIPTOR.message_types_by_name['EnvInitialInput'] = _ENVINITIALINPUT
DESCRIPTOR.message_types_by_name['EnvInitialOutput'] = _ENVINITIALOUTPUT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObservationSet = _reflection.GeneratedProtocolMessageType('ObservationSet', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATIONSET,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ObservationSet)
  })
_sym_db.RegisterMessage(ObservationSet)

ActionSet = _reflection.GeneratedProtocolMessageType('ActionSet', (_message.Message,), {
  'DESCRIPTOR' : _ACTIONSET,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ActionSet)
  })
_sym_db.RegisterMessage(ActionSet)

EnvRunTrialInput = _reflection.GeneratedProtocolMessageType('EnvRunTrialInput', (_message.Message,), {
  'DESCRIPTOR' : _ENVRUNTRIALINPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvRunTrialInput)
  })
_sym_db.RegisterMessage(EnvRunTrialInput)

EnvRunTrialOutput = _reflection.GeneratedProtocolMessageType('EnvRunTrialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ENVRUNTRIALOUTPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvRunTrialOutput)
  })
_sym_db.RegisterMessage(EnvRunTrialOutput)

EnvInitialInput = _reflection.GeneratedProtocolMessageType('EnvInitialInput', (_message.Message,), {
  'DESCRIPTOR' : _ENVINITIALINPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvInitialInput)
  })
_sym_db.RegisterMessage(EnvInitialInput)

EnvInitialOutput = _reflection.GeneratedProtocolMessageType('EnvInitialOutput', (_message.Message,), {
  'DESCRIPTOR' : _ENVINITIALOUTPUT,
  '__module__' : 'cogment.api.environment_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.EnvInitialOutput)
  })
_sym_db.RegisterMessage(EnvInitialOutput)



_ENVIRONMENTSP = _descriptor.ServiceDescriptor(
  name='EnvironmentSP',
  full_name='cogmentAPI.EnvironmentSP',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=933,
  serialized_end=1093,
  methods=[
  _descriptor.MethodDescriptor(
    name='RunTrial',
    full_name='cogmentAPI.EnvironmentSP.RunTrial',
    index=0,
    containing_service=None,
    input_type=_ENVRUNTRIALINPUT,
    output_type=_ENVRUNTRIALOUTPUT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='cogmentAPI.EnvironmentSP.Version',
    index=1,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._VERSIONREQUEST,
    output_type=cogment_dot_api_dot_common__pb2._VERSIONINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENVIRONMENTSP)

DESCRIPTOR.services_by_name['EnvironmentSP'] = _ENVIRONMENTSP

# @@protoc_insertion_point(module_scope)



# Copyright 2021 AI Redefined Inc. <dev+cogment@ai-r.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import grpc.aio  # type: ignore

import cogment.api.datalog_pb2_grpc as datalog_grpc_api
import cogment.api.common_pb2 as common_api
import cogment.api.datalog_pb2 as datalog_api

from cogment.control import TrialState
from cogment.datalog import DatalogSession
from cogment.errors import CogmentError
from cogment.utils import list_versions
from cogment.session import RecvObservation, RecvAction, RecvMessage, RecvReward

import logging
import asyncio


class LogParams:
    """Class representing the parameters of a trial for the datalog service."""

    def __init__(self, cog_settings):
        self.max_steps = None
        self.max_inactivity = None
        self.datalog = None
        self.environment = None
        self.nb_actors = None

        self._actor_indexes = None
        self._raw_params = None
        self._settings = cog_settings

    def __str__(self):
        result = f"LogParams: {self._raw_params}"
        return result

    # Type of serialized data being produced and consumed by this class.
    # This is dependent on all the underlying protobuf messages used to
    # serialize/deserialize, and should be incremented if any of them changes in
    # a backward or forward incompatible way. API 1.0 could be considered a type 0 or 1.
    # Current dependencies: TrialParams, DatalogParams, EnvironmentParams, ActorParams,
    #                       TrialConfig, EnvironmentConfig, ActorConfig
    def get_serialization_type(self):
        return 2

    def _set(self, raw_params):
        self._set_from_params(raw_params)

    def serialize(self):
        if self._raw_params is None:
            raise CogmentError("Not set, cannot serialize")
        return self._raw_params.SerializeToString()

    def deserialize(self, raw_string):
        params = common_api.TrialParams()
        params.ParseFromString(raw_string)
        self._set_from_params(params)

    def _set_from_params(self, raw_params):
        if type(raw_params) != common_api.TrialParams:
            raise CogmentError(f"Wrong type of parameters provided [{type(raw_params)}]")

        self.max_steps = raw_params.max_steps
        self.max_inactivity = raw_params.max_inactivity

        self.datalog = {
            "endpoint": raw_params.datalog.endpoint,
            "exclude": raw_params.datalog.exclude_fields,
        }

        self.environment = {
            "name": raw_params.environment.name,
            "endpoint": raw_params.environment.endpoint,
            "implementation": raw_params.environment.implementation,
        }

        self.nb_actors = len(raw_params.actors)
        self._actor_indexes = None
        self._raw_params = raw_params

    def get_trial_config(self):
        config = None
        if self._raw_params.HasField("trial_config"):
            config = self._settings.trial.config_type()
            config.ParseFromString(self._raw_params.trial_config.content)

        return config

    def get_environment_config(self):
        config = None
        if(self._raw_params.environment.HasField("config")):
            config = self._settings.environment.config_type()
            config.ParseFromString(self._raw_params.environment.config.content)

        return config

    def get_actor_index(self, actor_name):
        if self._actor_indexes is not None:
            return self._actor_indexes.get(actor_name)
        else:
            indexes = {}
            for index, actor in enumerate(self._raw_params.actors):
                indexes[actor.name] = index
            self._actor_indexes = indexes
            return indexes.get(actor_name)

    def get_actor_name(self, actor_index):
        return self._raw_params.actors[actor_index].name

    def get_actor(self, actor_index):
        actor = self._raw_params.actors[actor_index]

        actor_config = None
        a_c = self._settings.actor_classes.__getattribute__(actor.actor_class)
        if actor.HasField("config"):
            actor_config = a_c.config_type()
            actor_config.ParseFromString(actor.config.content)

        actor_data = {
            "name": actor.name,
            "actor_class": actor.actor_class,
            "endpoint": actor.endpoint,
            "implementation": actor.implementation,
            "config": actor_config,
            "action_space": a_c.action_space,
            "observation_space": a_c.observation_space,
        }

        return actor_data


class LogSample:
    """Class representing a trial sample for the datalog service."""

    def __init__(self, params):
        self.tick_id = None
        self.timestamp = None
        self.state = None
        self.events = None

        self._raw_sample = None

        if type(params) != LogParams:
            raise CogmentError(f"Wrong type of params provided [{type(params)}]")
        self._params = params

    def __str__(self):
        result = f"LogSample: {self._raw_sample}"
        return result

    # Type of serialized data being produced and consumed by this class.
    # This is dependent on all the underlying protobuf messages used to
    # serialize/deserialize, and should be incremented if any of them changes in
    # a backward or forward incompatible way. API 1.0 could be considered a type 0 or 1.
    # Current dependencies: DatalogSample, SampleInfo, TrialState, ObservationSet,
    #                       Action, Reward, RewardSource, Message
    def get_serialization_type(self):
        return 2

    def _set(self, raw_sample):
        self._set_from_sample(raw_sample)

    def serialize(self):
        if self._raw_sample is None:
            raise CogmentError("Not set, nothing to serialize")
        return self._raw_sample.SerializeToString()

    def deserialize(self, raw_string):
        sample = datalog_api.DatalogSample()
        sample.ParseFromString(raw_string)
        self._set_from_sample(sample)

    def _set_from_sample(self, sample):
        if type(sample) != datalog_api.DatalogSample:
            raise CogmentError(f"Wrong type of sample provided [{type(sample)}]")
        if sample.HasField("info"):
            self.state = TrialState(sample.info.state)
            self.tick_id = sample.info.tick_id
            self.timestamp = sample.info.timestamp
            self.events = sample.info.special_events
        else:
            self.state = None
            self.tick_id = None
            self.timestamp = None
            self.events = None

        self._raw_sample = sample

    def all_actor_names(self):
        for index in range(self._params.nb_actors):
            yield self._params.get_actor_name(index)

    def get_action(self, actor):
        if isinstance(actor, int):
            actor_index = actor
        elif isinstance(actor, str):
            actor_index = self._params.get_actor_index(actor)
        else:
            raise CogmentError(f"Wrong type of actor parameter [{type(actor)}]: must be int or str")
        if actor_index is None or actor_index < 0 or actor_index >= self._params.nb_actors:
            raise CogmentError(f"Invalid actor [{actor}] [{actor_index}]")

        if len(self._raw_sample.actions) > 0:
            data = self._raw_sample.actions[actor_index]
            action = self._params.get_actor(actor_index)["action_space"]()
            action.ParseFromString(data.content)
            return RecvAction(actor_index, data, action)
        else:
            return None

    def get_observation(self, actor):
        if isinstance(actor, int):
            actor_index = actor
        elif isinstance(actor, str):
            actor_index = self._params.get_actor_index(actor)
        else:
            raise CogmentError(f"Wrong type of actor parameter [{type(actor)}]: must be int or str")
        if actor_index is None or actor_index < 0 or actor_index >= self._params.nb_actors:
            raise CogmentError(f"Invalid actor [{actor}] [{actor_index}]")

        if self._raw_sample.HasField("observations"):
            data = self._raw_sample.observations
            obs_index = data.actors_map[actor_index]
            obs_content = data.observations[obs_index]
            obs = self._params.get_actor(actor_index)["observation_space"]()
            obs.ParseFromString(obs_content)
            return RecvObservation(data, obs)
        else:
            return None

    def all_rewards(self):
        for rew in self._raw_sample.rewards:
            yield RecvReward(rew)

    def all_messages(self):
        for msg in self._raw_sample.messages:
            yield RecvMessage(msg)


async def _read_sample(context, session, settings):
    try:
        while True:
            request = await context.read()

            if request == grpc.aio.EOF:
                logging.info(f"The orchestrator disconnected from LogExpoterService.")
                break

            elif request.HasField("sample"):
                trial_ended = (request.sample.info.state == common_api.TrialState.ENDED)
                sample = LogSample(session.trial_params)
                sample._set(request.sample)
                session._new_sample(sample)
                if trial_ended:
                    logging.debug("Last log sample received for trial")
                    break
            else:
                logging.warning(f"Invalid request received from the orchestrator : {request}")

    except asyncio.CancelledError as exc:
        logging.debug(f"DatalogServicer '_read_sample' coroutine cancelled: [{exc}]")
        raise

    except Exception:
        logging.exception("_read_sample")
        raise

    # Exit the loop
    session._new_sample(None)


class DatalogServicer(datalog_grpc_api.DatalogSPServicer):
    """Internal datalog servicer class."""

    def __init__(self, impl, cog_settings):
        self._impl = impl
        self.__cog_settings = cog_settings
        logging.info("Datalog Service started")

    # Override
    async def RunTrialDatalog(self, request_iterator, context):
        reader_task = None
        try:
            metadata = dict(context.invocation_metadata())
            trial_id = metadata["trial-id"]
            user_id = metadata["user-id"]

            request = await context.read()

            if not request.HasField("trial_params"):
                raise CogmentError(f"Initial logging request for [{trial_id}] does not contain parameters.")

            trial_params = LogParams(self.__cog_settings)
            trial_params._set(request.trial_params)

            session = DatalogSession(self._impl, trial_id, user_id, trial_params)
            user_task = session._start_user_task()

            reader_task = asyncio.create_task(_read_sample(context, session, self.__cog_settings))

            normal_return = await user_task

            if normal_return:
                logging.debug(f"User datalog implementation returned")
            else:
                logging.debug(f"User datalog implementation was cancelled")

        except asyncio.CancelledError as exc:
            logging.debug(f"Datalog implementation coroutine cancelled: [{exc}]")

        except Exception:
            logging.exception("RunTrialDatalog")
            raise

        finally:
            if reader_task is not None:
                reader_task.cancel()

    # Override
    async def Version(self, request, context):
        try:
            return list_versions()
        except Exception:
            logging.exception("Version")
            raise



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/hooks.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cogment/api/hooks.proto',
  package='cogmentAPI',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17\x63ogment/api/hooks.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"9\n\x0ePreTrialParams\x12\'\n\x06params\x18\x01 \x01(\x0b\x32\x17.cogmentAPI.TrialParams2\x98\x01\n\x0cTrialHooksSP\x12\x46\n\nOnPreTrial\x12\x1a.cogmentAPI.PreTrialParams\x1a\x1a.cogmentAPI.PreTrialParams\"\x00\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x62\x06proto3'
  ,
  dependencies=[cogment_dot_api_dot_common__pb2.DESCRIPTOR,])




_PRETRIALPARAMS = _descriptor.Descriptor(
  name='PreTrialParams',
  full_name='cogmentAPI.PreTrialParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='cogmentAPI.PreTrialParams.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=122,
)

_PRETRIALPARAMS.fields_by_name['params'].message_type = cogment_dot_api_dot_common__pb2._TRIALPARAMS
DESCRIPTOR.message_types_by_name['PreTrialParams'] = _PRETRIALPARAMS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PreTrialParams = _reflection.GeneratedProtocolMessageType('PreTrialParams', (_message.Message,), {
  'DESCRIPTOR' : _PRETRIALPARAMS,
  '__module__' : 'cogment.api.hooks_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.PreTrialParams)
  })
_sym_db.RegisterMessage(PreTrialParams)



_TRIALHOOKSSP = _descriptor.ServiceDescriptor(
  name='TrialHooksSP',
  full_name='cogmentAPI.TrialHooksSP',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=125,
  serialized_end=277,
  methods=[
  _descriptor.MethodDescriptor(
    name='OnPreTrial',
    full_name='cogmentAPI.TrialHooksSP.OnPreTrial',
    index=0,
    containing_service=None,
    input_type=_PRETRIALPARAMS,
    output_type=_PRETRIALPARAMS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='cogmentAPI.TrialHooksSP.Version',
    index=1,
    containing_service=None,
    input_type=cogment_dot_api_dot_common__pb2._VERSIONREQUEST,
    output_type=cogment_dot_api_dot_common__pb2._VERSIONINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TRIALHOOKSSP)

DESCRIPTOR.services_by_name['TrialHooksSP'] = _TRIALHOOKSSP

# @@protoc_insertion_point(module_scope)
